/* 
driver 2: load specified matrix and run full QR,
partial pivoted QR and blocked randQB algorithms
*/

#include "rank_revealing_algorithms_intel_mkl.h"


int main()
{
    int i, j, m, n, k, kstep, frank, tn, ntrials;
    double normM,normU,normS,normV,normP,percent_error;
    mat *M, *Qk, *Rk, *P, *Porig, *QkRk, *QkRkPt, *Q, *B, *QB;
    vec *I, *col_vec;
    //time_t start_time, end_time;
    struct timeval start_timeval, end_timeval;
    double elapsed_secsQR1, elapsed_secsQR2, elapsed_secsQB0, elapsed_secsQB1, elapsed_secsQB2, elapsed_secsQBb0, elapsed_secsQBb1, elapsed_secsQBb2;
    double elapsed_secsQR1_sum, elapsed_secsQR2_sum, elapsed_secsQB0_sum, elapsed_secsQB1_sum, elapsed_secsQB2_sum, elapsed_secsQBb0_sum, elapsed_secsQBb1_sum, elapsed_secsQBb2_sum;
    FILE *fp;

    //char *M_file = "data/A_mat1.bin";
    char *M_file = "data/A_mat4.bin";
    
    printf("loading matrix from %s\n", M_file);
    M = matrix_load_from_binary_file(M_file);
    m = M->nrows;
    n = M->ncols;
    printf("sizes of M are %d by %d\n", m, n);


    // set up vars
    k = 10;
    kstep = 5;


    // make mats
    QB = matrix_new(m,n);
    QkRk = matrix_new(m,n);
    QkRkPt = matrix_new(m,n);
    Porig = matrix_new(n,n);
    P = matrix_new(n,n);


    // full pivoted QR ----> 
    printf("do full QR..\n");
    gettimeofday(&start_timeval, NULL);
    pivotedQR_mkl(M, &Qk, &Rk, &I);
    gettimeofday(&end_timeval, NULL);
    elapsed_secsQR1 = get_seconds_frac(start_timeval,end_timeval);
    printf("full QR elapsed time: about %4.4f seconds\n", get_seconds_frac(start_timeval,end_timeval));


    printf("building P..\n");
    initialize_identity_matrix(Porig);
    #pragma omp parallel shared(Porig,P,I) private(i,col_vec) 
    {
    #pragma omp for
    for(i=0; i<Porig->ncols; i++){
        col_vec = vector_new(Porig->ncols);
        matrix_get_col(Porig,vector_get_element(I,i),col_vec);
        matrix_set_col(P,i,col_vec);
        vector_delete(col_vec);
    }
    }
    
    matrix_matrix_mult(Qk,Rk,QkRk);
    matrix_matrix_transpose_mult(QkRk,P,QkRkPt);
    percent_error = get_percent_error_between_two_mats(M,QkRkPt);
    //printf("output rank: %d\n", frank);
    printf("percent_error between M and QkRkPt = %f\n", percent_error);    
    printf("QR elapsed time: about %4.4f seconds\n", get_seconds_frac(start_timeval,end_timeval));
    matrix_delete(Qk); matrix_delete(Rk); vector_delete(I);



    printf("do partial pivoted QR..\n");
    gettimeofday(&start_timeval, NULL);
    pivoted_QR_of_specified_rank(M, k, &frank, &Qk, &Rk, &I);
    gettimeofday(&end_timeval, NULL);
    elapsed_secsQR2 = get_seconds_frac(start_timeval,end_timeval);


    printf("building P..\n");
    initialize_identity_matrix(Porig);
    #pragma omp parallel shared(Porig,P,I) private(i,col_vec) 
    {
    #pragma omp for
    for(i=0; i<Porig->ncols; i++){
        col_vec = vector_new(Porig->ncols);
        matrix_get_col(Porig,vector_get_element(I,i),col_vec);
        matrix_set_col(P,i,col_vec);
        vector_delete(col_vec);
    }
    }
    

    matrix_matrix_mult(Qk,Rk,QkRk);
    matrix_matrix_transpose_mult(QkRk,P,QkRkPt);
    percent_error = get_percent_error_between_two_mats(M,QkRkPt);
    //printf("output rank: %d\n", frank);
    printf("percent_error between M and QkRkPt = %f\n", percent_error);    
    printf("QR elapsed time: about %4.4f seconds\n", get_seconds_frac(start_timeval,end_timeval));
    matrix_delete(Qk); matrix_delete(Rk); vector_delete(I);



           
    // do QB blocked ----->
    printf("doing blocked QB p = 0..\n");
    gettimeofday(&start_timeval, NULL);
    randQB_pb(M, kstep, (int)(k/kstep), 0, &Q, &B);
    gettimeofday(&end_timeval, NULL);
    elapsed_secsQBb0 = get_seconds_frac(start_timeval,end_timeval);
    matrix_matrix_mult(Q,B,QB);
    percent_error = get_percent_error_between_two_mats(M,QB);
    printf("blocked percent_error between M and QB = %f\n", percent_error);
    printf("blocked elapsed time = %f\n", elapsed_secsQBb0);
    matrix_delete(Q); matrix_delete(B);


    printf("doing blocked QB p = 1..\n");
    gettimeofday(&start_timeval, NULL);
    randQB_pb(M, kstep, (int)(k/kstep), 1, &Q, &B);
    gettimeofday(&end_timeval, NULL);
    elapsed_secsQBb1 = get_seconds_frac(start_timeval,end_timeval);
    matrix_matrix_mult(Q,B,QB);
    percent_error = get_percent_error_between_two_mats(M,QB);
    printf("blocked percent_error between M and QB = %f\n", percent_error);
    printf("blocked elapsed time = %f\n", elapsed_secsQBb1);
    matrix_delete(Q); matrix_delete(B);


    printf("doing blocked QB p = 2..\n");
    gettimeofday(&start_timeval, NULL);
    randQB_pb(M, kstep, (int)(k/kstep), 2, &Q, &B);
    gettimeofday(&end_timeval, NULL);
    elapsed_secsQBb2 = get_seconds_frac(start_timeval,end_timeval);
    matrix_matrix_mult(Q,B,QB);
    percent_error = get_percent_error_between_two_mats(M,QB);
    printf("blocked percent_error between M and QB = %f\n", percent_error);
    printf("blocked elapsed time = %f\n", elapsed_secsQBb2);
    matrix_delete(Q); matrix_delete(B);


    // delete working mats and vectors for this n
    matrix_delete(QkRk);
    matrix_delete(QkRkPt);
    matrix_delete(QB);
    matrix_delete(Porig);
    matrix_delete(P);


    return 0;
}

